# Test reading 8-bit image values as list and matrix using the pawt package.
#
# Copyright: 2021-2022 Paul Obermeier (obermeier@poSoft.de)
# Distributed under BSD license.

if { [file exists "TestUtils.tcl"] } {
    source "TestUtils.tcl"
}

package require pawt

if { ! [pawt::HavePkg "Img"] } {
    puts "Error: pawt test suite needs the Img extension."
    exit 1
}

if { ! [file isdirectory "testOut"] } {
    file mkdir "testOut"
}

set testCase "01_Basic-ListMatrix-byte"
pawtTest AddImgDicts -title $testCase

puts "\nCreate 1-channel image from list with scanorder TopDown. Get as list with scanorder TopDown."
set imgDict [pawt CreateImage -pixeltype "byte" -width 6 -height 2 -scanorder "TopDown" -list [pawtTest GetGreyListTD-byte]]
set imgAsList [pawt GetImageAsList imgDict -scanorder "TopDown"]
pawtTest CheckList [pawtTest GetGreyListTD-byte] $imgAsList "Check lists"

puts "\nCreate 1-channel image from list with scanorder BottomUp. Get as list with scanorder BottomUp."
set imgDict [pawt CreateImage -pixeltype "byte" -width 6 -height 2 -scanorder "BottomUp" -list [pawtTest GetGreyListTD-byte]]
set imgAsList [pawt GetImageAsList imgDict -scanorder "BottomUp"]
pawtTest CheckList [pawtTest GetGreyListTD-byte] $imgAsList "Check lists"

puts "\nCreate 1-channel image from list with scanorder TopDown. Get as list with scanorder BottomUp."
set imgDict [pawt CreateImage -pixeltype "byte" -width 6 -height 2 -scanorder "TopDown" -list [pawtTest GetGreyListTD-byte]]
set imgAsList [pawt GetImageAsList imgDict -scanorder "BottomUp"]
pawtTest CheckList [pawtTest GetGreyListBU-byte] $imgAsList "Check lists"

puts "\nCreate 1-channel image from list with scanorder BottomUp. Get as list with scanorder TopDown."
set imgDict [pawt CreateImage -pixeltype "byte" -width 6 -height 2 -scanorder "BottomUp" -list [pawtTest GetGreyListTD-byte]]
set imgAsList [pawt GetImageAsList imgDict -scanorder "TopDown"]
pawtTest CheckList [pawtTest GetGreyListBU-byte] $imgAsList "Check lists"


puts "\nCreate 1-channel image from matrix with scanorder TopDown. Get as matrix with scanorder TopDown."
set imgDict [pawt CreateImage -pixeltype "byte" -width 6 -height 2 -scanorder "TopDown" -matrix [pawtTest GetGreyMatrixTD-byte]]
set imgAsMatrix [pawt GetImageAsMatrix imgDict -scanorder "TopDown"]
pawtTest CheckMatrix [pawtTest GetGreyMatrixTD-byte] $imgAsMatrix "Check matrices"

puts "\nCreate 1-channel image from matrix with scanorder BottomUp. Get as matrix with scanorder BottomUp."
set imgDict [pawt CreateImage -pixeltype "byte" -width 6 -height 2 -scanorder "BottomUp" -matrix [pawtTest GetGreyMatrixTD-byte]]
set imgAsMatrix [pawt GetImageAsMatrix imgDict -scanorder "BottomUp"]
pawtTest CheckMatrix [pawtTest GetGreyMatrixTD-byte] $imgAsMatrix "Check matrices"

puts "\nCreate 1-channel image from matrix with scanorder TopDown. Get as matrix with scanorder BottomUp."
set imgDict [pawt CreateImage -pixeltype "byte" -width 6 -height 2 -scanorder "TopDown" -matrix [pawtTest GetGreyMatrixTD-byte]]
set imgAsMatrix [pawt GetImageAsMatrix imgDict -scanorder "BottomUp"]
pawtTest CheckMatrix [pawtTest GetGreyMatrixBU-byte] $imgAsMatrix "Check matrices"

puts "\nCreate 1-channel image from matrix with scanorder BottomUp. Get as matrix with scanorder TopDown."
set imgDict [pawt CreateImage -pixeltype "byte" -width 6 -height 2 -scanorder "BottomUp" -matrix [pawtTest GetGreyMatrixTD-byte]]
set imgAsMatrix [pawt GetImageAsMatrix imgDict -scanorder "TopDown"]
pawtTest CheckMatrix [pawtTest GetGreyMatrixBU-byte] $imgAsMatrix "Check matrices"

set minMaxList  [pawt GetImageMinMax     imgDict]
set meanStdList [pawt GetImageMeanStdDev imgDict]
pawtTest CheckNumber 0          [lindex $minMaxList 0]   "Minimum"
pawtTest CheckNumber 255        [lindex $minMaxList 1]   "Maximum"
# Mean and StdDev reference values computed with Octave.
pawtTest CheckFloat  125.83333  [lindex $meanStdList 0]  "Mean"               5
pawtTest CheckFloat   90.47434  [lindex $meanStdList 1]  "Standard Deviation" 5

pawtTest CheckString "0"        [pawt GetImageMinAsString    imgDict]    "Minimum String"
pawtTest CheckString "255"      [pawt GetImageMaxAsString    imgDict]    "Maximum String"
pawtTest CheckString  125.8333  [pawt GetImageMeanAsString   imgDict]    "Mean String"
pawtTest CheckString  90.4743   [pawt GetImageStdDevAsString imgDict]    "StdDev String"
pawtTest CheckString  125.8     [pawt GetImageMeanAsString   imgDict 1]  "Mean String (Prec: 1)"
pawtTest CheckString  90.5      [pawt GetImageStdDevAsString imgDict 1]  "StdDev String (Prec: 1)"



puts "\nCreate 3-channel image from list with scanorder TopDown. Get as list with scanorder TopDown."
set imgDict [pawt CreateImage -numchan 3 -pixeltype "byte" -width 6 -height 2 -scanorder "TopDown" -list [pawtTest GetRgbListTD-byte]]
set imgAsList [pawt GetImageAsList imgDict -scanorder "TopDown"]
pawtTest CheckList [pawtTest GetRgbListTD-byte] $imgAsList "Check lists"

puts "\nCreate 3-channel image from list with scanorder BottomUp. Get as list with scanorder BottomUp."
set imgDict [pawt CreateImage -numchan 3 -pixeltype "byte" -width 6 -height 2 -scanorder "BottomUp" -list [pawtTest GetRgbListTD-byte]]
set imgAsList [pawt GetImageAsList imgDict -scanorder "BottomUp"]
pawtTest CheckList [pawtTest GetRgbListTD-byte] $imgAsList "Check lists"

puts "\nCreate 3-channel image from list with scanorder TopDown. Get as list with scanorder BottomUp."
set imgDict [pawt CreateImage -numchan 3 -pixeltype "byte" -width 6 -height 2 -scanorder "TopDown" -list [pawtTest GetRgbListTD-byte]]
set imgAsList [pawt GetImageAsList imgDict -scanorder "BottomUp"]
pawtTest CheckList [pawtTest GetRgbListBU-byte] $imgAsList "Check lists"

puts "\nCreate 3-channel image from list with scanorder BottomUp. Get as list with scanorder TopDown."
set imgDict [pawt CreateImage -numchan 3 -pixeltype "byte" -width 6 -height 2 -scanorder "BottomUp" -list [pawtTest GetRgbListTD-byte]]
set imgAsList [pawt GetImageAsList imgDict -scanorder "TopDown"]
pawtTest CheckList [pawtTest GetRgbListBU-byte] $imgAsList "Check lists"


puts "\nCreate 3-channel image from matrix with scanorder TopDown. Get as matrix with scanorder TopDown."
set imgDict [pawt CreateImage -numchan 3 -pixeltype "byte" -width 6 -height 2 -scanorder "TopDown" -matrix [pawtTest GetRgbMatrixTD-byte]]
set imgAsMatrix [pawt GetImageAsMatrix imgDict -scanorder "TopDown"]
pawtTest CheckMatrix [pawtTest GetRgbMatrixTD-byte] $imgAsMatrix "Check matrices"

puts "\nCreate 3-channel image from matrix with scanorder BottomUp. Get as matrix with scanorder BottomUp."
set imgDict [pawt CreateImage -numchan 3 -pixeltype "byte" -width 6 -height 2 -scanorder "BottomUp" -matrix [pawtTest GetRgbMatrixTD-byte]]
set imgAsMatrix [pawt GetImageAsMatrix imgDict -scanorder "BottomUp"]
pawtTest CheckMatrix [pawtTest GetRgbMatrixTD-byte] $imgAsMatrix "Check matrices"

puts "\nCreate 3-channel image from matrix with scanorder TopDown. Get as matrix with scanorder BottomUp."
set imgDict [pawt CreateImage -numchan 3 -pixeltype "byte" -width 6 -height 2 -scanorder "TopDown" -matrix [pawtTest GetRgbMatrixTD-byte]]
set imgAsMatrix [pawt GetImageAsMatrix imgDict -scanorder "BottomUp"]
pawtTest CheckMatrix [pawtTest GetRgbMatrixBU-byte] $imgAsMatrix "Check matrices"

puts "\nCreate 3-channel image from matrix with scanorder BottomUp. Get as matrix with scanorder TopDown."
set imgDict [pawt CreateImage -numchan 3 -pixeltype "byte" -width 6 -height 2 -scanorder "BottomUp" -matrix [pawtTest GetRgbMatrixTD-byte]]
set imgAsMatrix [pawt GetImageAsMatrix imgDict -scanorder "TopDown"]
pawtTest CheckMatrix [pawtTest GetRgbMatrixBU-byte] $imgAsMatrix "Check matrices"

set minMaxList  [pawt GetImageMinMax     imgDict]
set meanStdList [pawt GetImageMeanStdDev imgDict]
pawtTest CheckNumber 0          [lindex $minMaxList 0]   "Minimum"
pawtTest CheckNumber 255        [lindex $minMaxList 1]   "Maximum"
# Mean and StdDev reference values computed with Octave.
pawtTest CheckFloat  106.33333  [lindex $meanStdList 0]  "Mean"               5
pawtTest CheckFloat  116.07387  [lindex $meanStdList 1]  "Standard Deviation" 5

pawtTest CheckString "0"        [pawt GetImageMinAsString    imgDict]    "Minimum String"
pawtTest CheckString "255"      [pawt GetImageMaxAsString    imgDict]    "Maximum String"
pawtTest CheckString  106.3333  [pawt GetImageMeanAsString   imgDict]    "Mean String"
pawtTest CheckString  116.0739  [pawt GetImageStdDevAsString imgDict]    "StdDev String"
pawtTest CheckString  106.3     [pawt GetImageMeanAsString   imgDict 1]  "Mean String (Prec: 1)"
pawtTest CheckString  116.1     [pawt GetImageStdDevAsString imgDict 1]  "StdDev String (Prec: 1)"

if { [lindex $argv 0] eq "auto" } {
    update
    after 500
    exit 0
}
