# Test handling 32-bit float RAW images using the pawt package.
# Note, that the RAW format supports storing rows in TopDown and BottomUp order.
#
# Copyright: 2021-2022 Paul Obermeier (obermeier@poSoft.de)
# Distributed under BSD license.

if { [file exists "TestUtils.tcl"] } {
    source "TestUtils.tcl"
}

package require pawt

if { ! [pawt::HavePkg "Img"] } {
    puts "Error: pawt test suite needs the Img extension."
    exit 1
}

if { ! [file isdirectory "testOut"] } {
    file mkdir "testOut"
}

set testCase "02_Format-Raw-float"

puts "\nCreate 1-channel float image"
set imgDict [pawt CreateImage -width 6 -height 2 -pixeltype "float" -numchan 1 -list [pawtTest GetGreyListTD-float]]

set outFilePath [file join "testOut" [format "%s_1chan_TopDown.raw" $testCase]]
puts "Write  1-channel float image [file tail $outFilePath] (TopDown)"
pawt::raw::WriteImageFile $imgDict $outFilePath -scanorder "TopDown"

puts "Read   1-channel float image [file tail $outFilePath]"
set rawDict1 [pawt::raw::ReadImageFile $outFilePath]
set rawImgAsList [pawt GetImageAsList rawDict1]
pawtTest CheckFloatList [pawtTest GetGreyListTD-float] $rawImgAsList "Check input vs. RAW-TopDown" 5
pawtTest AddImgDicts $imgDict "In: 1-channel float (TopDown)"  $rawDict1 "Out: 1-channel float (TopDown)"  -zoom 50 -title $testCase


set outFilePath [file join "testOut" [format "%s_1chan_BottomUp.raw" $testCase]]
puts "Write  1-channel float image [file tail $outFilePath] (BottomUp)"
pawt::raw::WriteImageFile $imgDict $outFilePath -scanorder "BottomUp"

puts "Read   1-channel float image [file tail $outFilePath]"
set rawDict2 [pawt::raw::ReadImageFile $outFilePath]
set rawImgAsList [pawt GetImageAsList rawDict2]
pawtTest CheckFloatList [pawtTest GetGreyListBU-float] $rawImgAsList "Check input vs. RAW-BottomUp" 5
pawtTest AddImgDicts $imgDict "In: 1-channel float (BottomUp)"  $rawDict2 "Out: 1-channel float (BottomUp)"  -zoom 50 -title $testCase


puts "\nCreate 3-channel float image"
set imgDict [pawt CreateImage -width 6 -height 2 -pixeltype "float" -numchan 3 -list [pawtTest GetRgbListTD-float]]

set outFilePath [file join "testOut" [format "%s_3chan_TopDown.raw" $testCase]]
puts "Write  3-channel float image [file tail $outFilePath] (TopDown)"
pawt::raw::WriteImageFile $imgDict $outFilePath -scanorder "TopDown"

puts "Read   3-channel float image [file tail $outFilePath]"
set rawDict3 [pawt::raw::ReadImageFile $outFilePath]
set rawImgAsList [pawt GetImageAsList rawDict3]
pawtTest CheckFloatList [pawtTest GetRgbListTD-float] $rawImgAsList "Check input vs. RAW-TopDown" 5
pawtTest AddImgDicts $imgDict "In: 3-channel float (TopDown)"  $rawDict3 "Out: 3-channel float (TopDown)"  -zoom 50 -title $testCase


set outFilePath [file join "testOut" [format "%s_3chan_BottomUp.raw" $testCase]]
puts "Write  3-channel float image [file tail $outFilePath] (BottomUp)"
pawt::raw::WriteImageFile $imgDict $outFilePath -scanorder "BottomUp"

puts "Read   3-channel float image [file tail $outFilePath]"
set rawDict4 [pawt::raw::ReadImageFile $outFilePath]
set rawImgAsList [pawt GetImageAsList rawDict4]
pawtTest CheckFloatList [pawtTest GetRgbListBU-float] $rawImgAsList "Check input vs. RAW-BottomUp" 5
pawtTest AddImgDicts $imgDict "In: 3-channel float (BottomUp)"  $rawDict4 "Out: 3-channel float (BottomUp)"  -zoom 50 -title $testCase


if { [lindex $argv 0] eq "auto" } {
    update
    after 500
    exit 0
}
